@@ -62,10 +62,10 @@
 #define MAX_SCROLL_TIME 0.4 /* seconds */
 #define SCROLL_DELAY 33     /* milliseconds */
 
-#define SCHEMES "(https?|ftps?|nntp|news|javascript|about|ghelp|apt|telnet|"\
+#define SCHEMES "(https?|s?ftps?|nntp|news|javascript|about|ghelp|apt|telnet|"\
 		"file|webcal|mailto)"
-#define BODY "([^\\ ]+)"
-#define END_BODY "([^\\ ]*[^,;\?><()\\ \"\\.])"
+#define BODY "([^\\ \\n]+)"
+#define END_BODY "([^\\ \\n]*[^,;\?><()\\ \"\\.\\n])"
 #define URI_REGEX "("SCHEMES"://"END_BODY")" \
 		  "|((mailto:)?"BODY"@"BODY"\\."END_BODY")"\
 		  "|((www|ftp)\\."END_BODY")"
@@ -323,7 +323,7 @@ chat_text_view_populate_popup (EmpathyChatTextView *view,
 						    &start, &end, FALSE);
 	    }
 	
-	if (G_STR_EMPTY (str)) {
+	if (EMP_STR_EMPTY (str)) {
 		g_free (str);
 		return;
 	}
@@ -637,7 +637,7 @@ empathy_chat_text_view_init (EmpathyChatTextView *view)
 	priv->buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 	priv->last_timestamp = 0;
 	priv->allow_scrolling = TRUE;
-	priv->smiley_manager = empathy_smiley_manager_new ();
+	priv->smiley_manager = empathy_smiley_manager_dup_singleton ();
 	
 	g_object_set (view,
 		      "wrap-mode", GTK_WRAP_WORD_CHAR,
@@ -762,7 +762,7 @@ chat_text_view_append_event (EmpathyChatView *view,
 
 
 	g_return_if_fail (EMPATHY_IS_CHAT_TEXT_VIEW (view));
-	g_return_if_fail (!G_STR_EMPTY (str));
+	g_return_if_fail (!EMP_STR_EMPTY (str));
 
 	bottom = chat_text_view_is_scrolled_down (text_view);
 	chat_text_view_maybe_trim_buffer (EMPATHY_CHAT_TEXT_VIEW (view));
@@ -856,7 +856,7 @@ chat_text_view_find_previous (EmpathyChatView *view,
 	
 	buffer = priv->buffer;
 	
-	if (G_STR_EMPTY (search_criteria)) {
+	if (EMP_STR_EMPTY (search_criteria)) {
 		if (priv->find_mark_previous) {
 			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
 			
@@ -971,7 +971,7 @@ chat_text_view_find_next (EmpathyChatView *view,
 	
 	buffer = priv->buffer;
 	
-	if (G_STR_EMPTY (search_criteria)) {
+	if (EMP_STR_EMPTY (search_criteria)) {
 		if (priv->find_mark_next) {
 			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
 			
@@ -1142,7 +1142,7 @@ chat_text_view_highlight (EmpathyChatView *view,
 					    &iter_start,
 					    &iter_end);
 	
-	if (G_STR_EMPTY (text)) {
+	if (EMP_STR_EMPTY (text)) {
 		return;
 	}
 	
