@@ -44,6 +44,8 @@ struct _SmileyManagerTree {
 
 G_DEFINE_TYPE (EmpathySmileyManager, empathy_smiley_manager, G_TYPE_OBJECT);
 
+static EmpathySmileyManager *manager_singleton = NULL;
+
 static SmileyManagerTree *
 smiley_manager_tree_new (gunichar c)
 {
@@ -115,12 +117,33 @@ smiley_manager_finalize (GObject *object)
 	g_slist_free (priv->smileys);
 }
 
+static GObject *
+smiley_manager_constructor (GType type,
+			    guint n_props,
+			    GObjectConstructParam *props)
+{
+	GObject *retval;
+
+	if (manager_singleton) {
+		retval = g_object_ref (manager_singleton);
+	} else {
+		retval = G_OBJECT_CLASS (empathy_smiley_manager_parent_class)->constructor
+			(type, n_props, props);
+
+		manager_singleton = EMPATHY_SMILEY_MANAGER (retval);
+		g_object_add_weak_pointer (retval, (gpointer *) &manager_singleton);
+	}
+
+	return retval;
+}
+
 static void
 empathy_smiley_manager_class_init (EmpathySmileyManagerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
 	object_class->finalize = smiley_manager_finalize;
+	object_class->constructor = smiley_manager_constructor;
 
 	g_type_class_add_private (object_class, sizeof (EmpathySmileyManagerPriv));
 }
@@ -134,22 +157,14 @@ empathy_smiley_manager_init (EmpathySmileyManager *manager)
 	manager->priv = priv;
 	priv->tree = smiley_manager_tree_new ('\0');
 	priv->smileys = NULL;
+
+	empathy_smiley_manager_load (manager);
 }
 
 EmpathySmileyManager *
-empathy_smiley_manager_new (void)
+empathy_smiley_manager_dup_singleton (void)
 {
-	static EmpathySmileyManager *manager = NULL;
-
-	if (!manager) {
-		manager = g_object_new (EMPATHY_TYPE_SMILEY_MANAGER, NULL);
-		g_object_add_weak_pointer (G_OBJECT (manager), (gpointer) &manager);
-		empathy_smiley_manager_load (manager);
-	} else {
-		g_object_ref (manager);
-	}
-
-	return manager;
+	return g_object_new (EMPATHY_TYPE_SMILEY_MANAGER, NULL);
 }
 
 static SmileyManagerTree *
@@ -227,8 +242,8 @@ empathy_smiley_manager_add (EmpathySmileyManager *manager,
 	va_list    var_args;
 
 	g_return_if_fail (EMPATHY_IS_SMILEY_MANAGER (manager));
-	g_return_if_fail (!G_STR_EMPTY (icon_name));
-	g_return_if_fail (!G_STR_EMPTY (first_str));
+	g_return_if_fail (!EMP_STR_EMPTY (icon_name));
+	g_return_if_fail (!EMP_STR_EMPTY (first_str));
 
 	smiley = empathy_pixbuf_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
 	if (smiley) {
@@ -249,7 +264,7 @@ empathy_smiley_manager_add_from_pixbuf (EmpathySmileyManager *manager,
 
 	g_return_if_fail (EMPATHY_IS_SMILEY_MANAGER (manager));
 	g_return_if_fail (GDK_IS_PIXBUF (smiley));
-	g_return_if_fail (!G_STR_EMPTY (first_str));
+	g_return_if_fail (!EMP_STR_EMPTY (first_str));
 
 	va_start (var_args, first_str);
 	smiley_manager_add_valist (manager, smiley, first_str, var_args);
@@ -347,3 +362,87 @@ empathy_smiley_manager_get_all (EmpathySmileyManager *manager)
 	return priv->smileys;
 }
 
+typedef struct {
+	EmpathySmileyManager *manager;
+	EmpathySmiley        *smiley;
+	EmpathySmileyMenuFunc func;    
+	gpointer              user_data;         
+} ActivateData;
+
+static void
+smiley_menu_data_free (gpointer  user_data,
+		       GClosure *closure)
+{
+	ActivateData *data = (ActivateData*) user_data;
+
+	g_object_unref (data->manager);
+	g_slice_free (ActivateData, data);
+}
+
+static void
+smiley_menu_activate_cb (GtkMenuItem *menuitem,
+			 gpointer     user_data)
+{
+	ActivateData *data = (ActivateData*) user_data;
+
+	data->func (data->manager, data->smiley, data->user_data);
+}
+
+GtkWidget *
+empathy_smiley_menu_new (EmpathySmileyManager *manager,
+			 EmpathySmileyMenuFunc func,
+			 gpointer              user_data)
+{
+	EmpathySmileyManagerPriv *priv = GET_PRIV (manager);
+	GSList                   *l;
+	GtkWidget                *menu;
+	gint                      x = 0;
+	gint                      y = 0;
+
+	g_return_val_if_fail (EMPATHY_IS_SMILEY_MANAGER (manager), NULL);
+	g_return_val_if_fail (func != NULL, NULL);
+
+	menu = gtk_menu_new ();
+
+	for (l = priv->smileys; l; l = l->next) {
+		EmpathySmiley *smiley;
+		GtkWidget     *item;
+		GtkWidget     *image;
+		ActivateData  *data;
+
+		smiley = l->data;
+		image = gtk_image_new_from_pixbuf (smiley->pixbuf);
+
+		item = gtk_image_menu_item_new_with_label ("");
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+
+		gtk_menu_attach (GTK_MENU (menu), item,
+				 x, x + 1, y, y + 1);
+
+		gtk_widget_set_tooltip_text (item, smiley->str);
+
+		data = g_slice_new (ActivateData);
+		data->manager = g_object_ref (manager);
+		data->smiley = smiley;
+		data->func = func;
+		data->user_data = user_data;
+
+		g_signal_connect_data (item, "activate",
+				       G_CALLBACK (smiley_menu_activate_cb),
+				       data,
+				       smiley_menu_data_free,
+				       0);
+
+		if (x > 3) {
+			y++;
+			x = 0;
+		} else {
+			x++;
+		}
+	}
+
+	gtk_widget_show_all (menu);
+
+	return menu;
+}
+
