@@ -16,9 +16,15 @@ GType empathy_contact_ready_get_type (void);
 #include <libempathy/empathy-debug.h>
 #define EMPATHY_TYPE_DEBUG_FLAGS empathy_debug_flags_get_type()
 GType empathy_debug_flags_get_type (void);
+#include <libempathy/empathy-dispatch-operation.h>
+#define EMPATHY_TYPE_DISPATCH_OPERATION_STATE empathy_dispatch_operation_state_get_type()
+GType empathy_dispatch_operation_state_get_type (void);
 #include <libempathy/empathy-tp-call.h>
 #define EMPATHY_TYPE_TP_CALL_STATUS empathy_tp_call_status_get_type()
 GType empathy_tp_call_status_get_type (void);
+#include <libempathy/empathy-tube-handler.h>
+#define EMPATHY_TYPE_TUBE_TYPE empathy_tube_type_get_type()
+GType empathy_tube_type_get_type (void);
 G_END_DECLS
 
 #endif /* __LIBEMPATHY_ENUM_TYPES_H__ */
