@@ -41,11 +41,11 @@ G_BEGIN_DECLS
 
 typedef struct _EmpathyLogManager      EmpathyLogManager;
 typedef struct _EmpathyLogManagerClass EmpathyLogManagerClass;
-typedef struct _EmpathyLogManagerPriv  EmpathyLogManagerPriv;
 typedef struct _EmpathyLogSearchHit    EmpathyLogSearchHit;
 
 struct _EmpathyLogManager {
 	GObject parent;
+	gpointer priv;
 };
 
 struct _EmpathyLogManagerClass {
@@ -61,7 +61,7 @@ struct _EmpathyLogSearchHit {
 };
 
 GType              empathy_log_manager_get_type              (void) G_GNUC_CONST;
-EmpathyLogManager *empathy_log_manager_new                   (void);
+EmpathyLogManager *empathy_log_manager_dup_singleton         (void);
 void               empathy_log_manager_add_message           (EmpathyLogManager *manager,
 							      const gchar       *chat_id,
 							      gboolean           chatroom,
