@@ -27,6 +27,8 @@
 #include <glib.h>
 #include <telepathy-glib/channel.h>
 
+#include "empathy-contact.h"
+
 G_BEGIN_DECLS
 
 #define EMPATHY_TYPE_TP_CALL (empathy_tp_call_get_type ())
@@ -38,18 +40,20 @@ G_BEGIN_DECLS
     EMPATHY_TYPE_TP_CALL))
 #define EMPATHY_IS_TP_CALL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), \
     EMPATHY_TYPE_TP_CALL))
-#define EMPATHY_TP_CALL_GET_CLASS(object) (G_TYPE_INSTANCE_GET_CLASS ((object), \
+#define EMPATHY_TP_CALL_GET_CLASS(object) \
+  (G_TYPE_INSTANCE_GET_CLASS ((object), \
     EMPATHY_TYPE_TP_CALL, EmpathyTpCallClass))
 
 typedef struct _EmpathyTpCall EmpathyTpCall;
 typedef struct _EmpathyTpCallClass EmpathyTpCallClass;
 
 struct _EmpathyTpCall {
-    GObject parent;
+  GObject parent;
+  gpointer priv;
 };
 
 struct _EmpathyTpCallClass {
-    GObjectClass parent_class;
+  GObjectClass parent_class;
 };
 
 typedef enum
@@ -70,6 +74,9 @@ typedef struct
 
 GType empathy_tp_call_get_type (void) G_GNUC_CONST;
 EmpathyTpCall *empathy_tp_call_new (TpChannel *channel);
+void empathy_tp_call_close (EmpathyTpCall *call);
+
+void empathy_tp_call_to (EmpathyTpCall *call, EmpathyContact *contact);
 
 void empathy_tp_call_accept_incoming_call (EmpathyTpCall *call);
 void empathy_tp_call_request_video_stream_direction (EmpathyTpCall *call,
@@ -83,6 +90,9 @@ void empathy_tp_call_add_output_video (EmpathyTpCall *call,
 void empathy_tp_call_set_output_volume (EmpathyTpCall *call, guint volume);
 void empathy_tp_call_mute_output (EmpathyTpCall *call, gboolean is_muted);
 void empathy_tp_call_mute_input (EmpathyTpCall *call, gboolean is_muted);
+void empathy_tp_call_start_tone (EmpathyTpCall *call, TpDTMFEvent event);
+void empathy_tp_call_stop_tone (EmpathyTpCall *call);
+gboolean empathy_tp_call_has_dtmf (EmpathyTpCall *call);
 
 G_END_DECLS
 
