@@ -33,7 +33,6 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnomeui/libgnomeui.h>
 
 #define DEBUG_FLAG EMPATHY_DEBUG_FT
 #include <libempathy/empathy-debug.h>
@@ -47,6 +46,8 @@
 
 #include "empathy-ft-manager.h"
 
+#include "extensions/extensions.h"
+
 /**
  * SECTION:empathy-ft-manager
  * @short_description: File transfer dialog
@@ -95,6 +96,8 @@ enum
 
 G_DEFINE_TYPE (EmpathyFTManager, empathy_ft_manager, G_TYPE_OBJECT);
 
+static EmpathyFTManager *manager_singleton = NULL;
+
 static gchar *
 ft_manager_format_interval (gint interval)
 {
@@ -435,6 +438,8 @@ ft_manager_remove_file_from_model (EmpathyFTManager *ft_manager,
 
   if (update_selection)
     gtk_tree_selection_select_iter (selection, &iter);
+
+  empathy_tp_file_close (tp_file);
 }
 
 static gboolean
@@ -485,22 +490,18 @@ ft_manager_add_tp_file_to_list (EmpathyFTManager *ft_manager,
   GtkTreeIter iter;
   GtkTreeSelection *selection;
   GtkTreePath *path;
-  GtkIconTheme *theme;
-  gchar *icon_name;
+  GIcon *icon;
   const gchar *content_type;
 
-  /* Get the icon name from the mime-type of the file.
-   * FIXME: Use g_content_type_get_icon instead of gnome_icon_lookup and drop
-   * libgnomeui. We need the "gicon" property on GtkCellRendererPixbuf which is
-   * in GTK+ 2.14 */
+  /* Get the icon name from the mime-type of the file. */
   content_type = empathy_tp_file_get_content_type (tp_file);
-  theme = gtk_icon_theme_get_default ();
-  icon_name = gnome_icon_lookup (theme, NULL, NULL, NULL, NULL,
-      content_type, GNOME_ICON_LOOKUP_FLAGS_NONE, NULL);
+  icon = g_content_type_get_icon (content_type);
 
   /* Append the ft in the store */
   gtk_list_store_insert_with_values (GTK_LIST_STORE (ft_manager->priv->model),
-      &iter, G_MAXINT, COL_FT_OBJECT, tp_file, COL_ICON, icon_name, -1);
+      &iter, G_MAXINT, COL_FT_OBJECT, tp_file, COL_ICON, icon, -1);
+
+  g_object_unref (icon);
 
   /* Insert the new row_ref in the hash table  */
   path = gtk_tree_model_get_path (GTK_TREE_MODEL (ft_manager->priv->model),
@@ -524,7 +525,6 @@ ft_manager_add_tp_file_to_list (EmpathyFTManager *ft_manager,
       G_CALLBACK (ft_manager_transferred_bytes_changed_cb), ft_manager);
 
   gtk_window_present (GTK_WINDOW (ft_manager->priv->window));
-  g_free (icon_name);
 }
 
 static void
@@ -556,7 +556,7 @@ ft_manager_open (EmpathyFTManager *ft_manager)
 
   uri = g_object_get_data (G_OBJECT (tp_file), "uri");
   DEBUG ("Opening URI: %s", uri);
-  empathy_url_show (uri);
+  empathy_url_show (GTK_WIDGET (ft_manager->priv->window), uri);
   g_object_unref (tp_file);
 }
 
@@ -687,7 +687,7 @@ ft_manager_build_ui (EmpathyFTManager *ft_manager)
   /* Setup the model */
   liststore = gtk_list_store_new (5,
       G_TYPE_INT,     /* percent */
-      G_TYPE_STRING,  /* icon */
+      G_TYPE_ICON,    /* icon */
       G_TYPE_STRING,  /* message */
       G_TYPE_STRING,  /* remaining */
       G_TYPE_OBJECT); /* ft_file */
@@ -721,7 +721,7 @@ ft_manager_build_ui (EmpathyFTManager *ft_manager)
       "stock-size", GTK_ICON_SIZE_DND, NULL);
   gtk_tree_view_column_pack_start (column, renderer, FALSE);
   gtk_tree_view_column_set_attributes (column, renderer,
-      "icon-name", COL_ICON, NULL);
+      "gicon", COL_ICON, NULL);
 
   renderer = gtk_cell_renderer_text_new ();
   g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
@@ -774,36 +774,51 @@ empathy_ft_manager_init (EmpathyFTManager *ft_manager)
   ft_manager_build_ui (ft_manager);
 }
 
+static GObject *
+empathy_ft_manager_constructor (GType type,
+                                guint n_props,
+                                GObjectConstructParam *props)
+{
+  GObject *retval;
+
+  if (manager_singleton)
+    {
+      retval = g_object_ref (manager_singleton);
+    }
+  else
+    {
+      retval = G_OBJECT_CLASS (empathy_ft_manager_parent_class)->constructor
+          (type, n_props, props);
+
+      manager_singleton = EMPATHY_FT_MANAGER (retval);
+      g_object_add_weak_pointer (retval, (gpointer *) &manager_singleton);
+    }
+
+  return retval;
+}
+
 static void
 empathy_ft_manager_class_init (EmpathyFTManagerClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->finalize = empathy_ft_manager_finalize;
+  object_class->constructor = empathy_ft_manager_constructor;
 
   g_type_class_add_private (object_class, sizeof (EmpathyFTManagerPriv));
 }
 
 /**
- * empathy_ft_manager_get_default:
+ * empathy_ft_manager_dup_singleton:
  *
- * Returns a new #EmpathyFTManager if there is not already one, or the existing
- * one if it exists.
+ * Returns a reference to the #EmpathyFTManager singleton object.
  *
  * Returns: a #EmpathyFTManager
  */
 EmpathyFTManager *
-empathy_ft_manager_get_default (void)
+empathy_ft_manager_dup_singleton (void)
 {
-  static EmpathyFTManager *manager_p = NULL;
-
-  if (!manager_p)
-    {
-      manager_p = g_object_new (EMPATHY_TYPE_FT_MANAGER, NULL);
-      g_object_add_weak_pointer (G_OBJECT (manager_p), (gpointer) &manager_p);
-    }
-
-  return manager_p;
+  return g_object_new (EMPATHY_TYPE_FT_MANAGER, NULL);
 }
 
 /**
