@@ -32,6 +32,7 @@
 #include <gdk/gdkx.h>
 
 #include <libebook/e-book.h>
+#include <libnotify/notify.h>
 
 #include <telepathy-glib/util.h>
 #include <libmissioncontrol/mc-account.h>
@@ -39,15 +40,16 @@
 
 #include <libempathy/empathy-idle.h>
 #include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-call-factory.h>
+#include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-dispatch-operation.h>
 #include <libempathy/empathy-tp-chat.h>
-#include <libempathy/empathy-tp-group.h>
+#include <libempathy/empathy-tp-call.h>
 
 #include <libempathy-gtk/empathy-conf.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 
-#include <extensions/extensions.h>
-
 #include "empathy-accounts-dialog.h"
 #include "empathy-main-window.h"
 #include "empathy-status-icon.h"
@@ -56,26 +58,31 @@
 #include "empathy-ft-manager.h"
 #include "bacon-message-connection.h"
 
+#include "extensions/extensions.h"
+
 #define DEBUG_FLAG EMPATHY_DEBUG_OTHER
 #include <libempathy/empathy-debug.h>
 
+#include <gst/gst.h>
+
 static BaconMessageConnection *connection = NULL;
 
 static void
-dispatch_channel_cb (EmpathyDispatcher *dispatcher,
-		     TpChannel         *channel,
-		     gpointer           user_data)
+dispatch_cb (EmpathyDispatcher *dispatcher,
+	     EmpathyDispatchOperation *operation,
+	     gpointer           user_data)
 {
-	const gchar *channel_type;
+	GQuark channel_type;
 
-	channel_type = tp_channel_get_channel_type (channel);
-	if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT)) {
+	channel_type = empathy_dispatch_operation_get_channel_type_id (operation);
+
+	if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_TEXT) {
 		EmpathyTpChat *tp_chat;
 		EmpathyChat   *chat = NULL;
 		const gchar   *id;
 
-		tp_chat = empathy_tp_chat_new (channel);
-		empathy_run_until_ready (tp_chat);
+		tp_chat = EMPATHY_TP_CHAT (
+			empathy_dispatch_operation_get_channel_wrapper (operation));
 
 		id = empathy_tp_chat_get_id (tp_chat);
 		if (!id) {
@@ -101,19 +108,22 @@ dispatch_channel_cb (EmpathyDispatcher *dispatcher,
 		}
 
 		empathy_chat_window_present_chat (chat);
-		g_object_unref (tp_chat);
-	}
-	else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA)) {
-		empathy_call_window_new (channel);
-	}
-	else if (!tp_strdiff (channel_type, EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER)) {
+
+		empathy_dispatch_operation_claim (operation);
+	} else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA) {
+		EmpathyCallFactory *factory;
+
+		factory = empathy_call_factory_get ();
+		empathy_call_factory_claim_channel (factory, operation);
+	} else if (channel_type == EMP_IFACE_QUARK_CHANNEL_TYPE_FILE_TRANSFER) {
 		EmpathyFTManager *ft_manager;
 		EmpathyTpFile    *tp_file;
 
-		ft_manager = empathy_ft_manager_get_default ();
-		tp_file = empathy_tp_file_new (channel);
+		ft_manager = empathy_ft_manager_dup_singleton ();
+		tp_file = EMPATHY_TP_FILE (
+			empathy_dispatch_operation_get_channel_wrapper (operation));
 		empathy_ft_manager_add_tp_file (ft_manager, tp_file);
-		g_object_unref (tp_file);
+		empathy_dispatch_operation_claim (operation);
 	}
 }
 
@@ -371,12 +381,37 @@ get_startup_timestamp ()
 	return (retval > 0) ? retval : 0;
 }
 
+static gboolean
+show_version_cb (const char *option_name,
+                 const char *value,
+                 gpointer data,
+                 GError **error)
+{
+	g_print ("%s\n", PACKAGE_STRING);
+
+	exit (EXIT_SUCCESS);
+
+	return FALSE;
+}
+
+static void
+new_call_handler_cb (EmpathyCallFactory *factory, EmpathyCallHandler *handler,
+	gboolean outgoing, gpointer user_data)
+{
+		EmpathyCallWindow *window;
+
+		window = empathy_call_window_new (handler);
+		gtk_widget_show (GTK_WIDGET (window));
+}
+
 int
 main (int argc, char *argv[])
 {
 	guint32            startup_timestamp;
 	EmpathyStatusIcon *icon;
 	EmpathyDispatcher *dispatcher;
+	EmpathyChatroomManager *chatroom_manager;
+	EmpathyCallFactory *call_factory;
 	GtkWidget         *window;
 	MissionControl    *mc;
 	EmpathyIdle       *idle;
@@ -398,18 +433,27 @@ main (int argc, char *argv[])
 		  0, G_OPTION_ARG_NONE, &accounts_dialog,
 		  N_("Show the accounts dialog"),
 		  NULL },
+		{ "version", 'v',
+		  G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, show_version_cb, NULL, NULL },
 		{ NULL }
 	};
 
 	/* Init */
 	g_thread_init (NULL);
+	empathy_init ();
+
 	if (!gtk_init_with_args (&argc, &argv,
-				 _("- Empathy Instant Messenger"),
+				 N_("- Empathy Instant Messenger"),
 				 options, GETTEXT_PACKAGE, &error)) {
 		g_warning ("Error in empathy init: %s", error->message);
 		return EXIT_FAILURE;
 	}
+
 	empathy_gtk_init ();
+	g_set_application_name (_(PACKAGE_NAME));
+
+	gst_init (&argc, &argv);
+
 	g_set_application_name (PACKAGE_NAME);
 	gtk_window_set_default_icon_name ("empathy");
 	textdomain (GETTEXT_PACKAGE);
@@ -449,10 +493,8 @@ main (int argc, char *argv[])
 		g_warning ("Cannot create the 'empathy' bacon connection.");
 	}
 
-	emp_cli_init ();
-
 	/* Setting up MC */
-	mc = empathy_mission_control_new ();
+	mc = empathy_mission_control_dup_singleton ();
 	g_signal_connect (mc, "ServiceEnded",
 			  G_CALLBACK (service_ended_cb),
 			  NULL);
@@ -473,7 +515,7 @@ main (int argc, char *argv[])
 	}
 
 	/* Setting up Idle */
-	idle = empathy_idle_new ();
+	idle = empathy_idle_dup_singleton ();
 	empathy_idle_set_auto_away (idle, TRUE);
 	use_nm_notify_cb (empathy_conf_get (), EMPATHY_PREFS_USE_NM, idle);
 	empathy_conf_notify_add (empathy_conf_get (), EMPATHY_PREFS_USE_NM,
@@ -502,10 +544,17 @@ main (int argc, char *argv[])
 	}
 
 	/* Handle channels */
-	dispatcher = empathy_dispatcher_new ();
-	g_signal_connect (dispatcher, "dispatch-channel",
-			  G_CALLBACK (dispatch_channel_cb),
-			  NULL);
+	dispatcher = empathy_dispatcher_dup_singleton ();
+	g_signal_connect (dispatcher, "dispatch", G_CALLBACK (dispatch_cb), NULL);
+
+	chatroom_manager = empathy_chatroom_manager_dup_singleton (NULL);
+	empathy_chatroom_manager_observe (chatroom_manager, dispatcher);
+
+	notify_init (_(PACKAGE_NAME));
+	/* Create the call factory */
+	call_factory = empathy_call_factory_initialise ();
+	g_signal_connect (G_OBJECT (call_factory), "new-call-handler",
+		G_CALLBACK (new_call_handler_cb), NULL);
 
 	gtk_main ();
 
@@ -515,6 +564,9 @@ main (int argc, char *argv[])
 	g_object_unref (idle);
 	g_object_unref (icon);
 	g_object_unref (dispatcher);
+	g_object_unref (chatroom_manager);
+
+	notify_uninit ();
 
 	return EXIT_SUCCESS;
 }
